#include <owl/dc.h>
#include <owl/gdiobjec.h>

#include <math.h>

#include "clock.h"

#define SPACING  5
#define TIMER_ID 100

DEFINE_RESPONSE_TABLE1(TClock, TControl)
  EV_WM_ERASEBKGND,
  EV_WM_PAINT,
  EV_WM_SIZE,
  EV_WM_TIMER,
END_RESPONSE_TABLE;

TClock::TClock(TWindow* parent, int id, const char* title, int x, int y,
               int w, int h, TModule* module)
:
 TControl(parent, id, title, x, y, w, h, module),
 x(0), y(0),
 r(0),
 scale(1.0),
 bkColor(TColor::Sys3dFace),
 clientRect(0, 0, 0, 0),
 memDC(0)
{
  GetTime();
}

TClock::TClock(TWindow* parent, int resourceId, TModule* module)
:
 TControl(parent, resourceId, module),
 x(0), y(0),
 r(0),
 scale(1.0),
 bkColor(TColor::Sys3dFace),
 clientRect(0, 0, 0, 0),
 memDC(0)
{
  GetTime();
}

TClock::~TClock()
{
}

void
TClock::SetupWindow()
{
  TControl::SetupWindow();

  TClientDC dc(*this);

  GetClientRect(clientRect);

  memDC = new TMemoryDC(dc);
  memDC->SelectObject(TBitmap(dc, clientRect.Width(), clientRect.Height()));

  AdjustParams();

  Plot(*memDC);

  SetTimer(TIMER_ID, 1000);  // Every one second
}

void
TClock::CleanupWindow()
{
  KillTimer(TIMER_ID);

  memDC->RestoreBitmap();
  delete memDC;

  TWindow::CleanupWindow();
}

void
TClock::EvPaint()
{
  TPaintDC dc(*this);
  dc.BitBlt(GetClientRect(), *memDC, TPoint(0, 0));
}

void
TClock::EvSize(uint sizeType, TSize& size)
{
  TControl::EvSize(sizeType, size);

  GetClientRect(clientRect);

  memDC->RestoreBitmap();

  TClientDC dc(*this);
  memDC->SelectObject(TBitmap(dc, clientRect.Width(), clientRect.Height()));

  AdjustParams();

  Plot(*memDC);

  Invalidate();
}

void
TClock::EvTimer(uint timerId)
{
  if (timerId == TIMER_ID) {
    GetTime();

    Plot(*memDC);

    Invalidate();
  }
}

void
TClock::AdjustParams()
{
  x = clientRect.Width () / 2;
  y = clientRect.Height() / 2;

  r = (x < y) ? x : y;

  scale = r / 100.0;

}

void
TClock::GetTime()
{
  SYSTEMTIME t;
  ::GetLocalTime(&t);

  h = t.wHour   % 12;
  m = t.wMinute % 60;
  s = t.wSecond % 60;
}

void
TClock::Plot(TDC& dc)
{
  int spacing = scale * SPACING;

  int cr = r;

  double angle;

  // Set background color
  //
  dc.FillRect(clientRect, bkColor);

  // Draw the outer circle
  //
  dc.SelectObject(TBrush(TColor(120, 160, 230)));

  dc.Ellipse(x - cr, y - cr, x + cr, y + cr);

  dc.RestoreBrush();

  // Draw the inner circle
  //
  cr -= 2 * spacing;

  dc.SelectObject(TBrush(TColor(80, 120, 190)));

  dc.Ellipse(x - cr, y - cr, x + cr, y + cr);

  dc.RestoreBrush();

  // Draw the lines indicating seconds
  //
  int r1 = cr - 2 * spacing;
  int r2 = cr -     spacing;

  angle = 0.0;

  dc.SelectObject(TPen(TColor::Black, 2, PS_SOLID));

  for (int i = 0; i < 12; i++) {
    dc.MoveTo(x + r1 * cos(angle), y + r1 * sin(angle));
    dc.LineTo(x + r2 * cos(angle), y + r2 * sin(angle));

    angle += M_PI / 6.0;
  }

  dc.RestorePen();

  // Draw the second handle
  //
  cr -= 3 * spacing;

  angle = M_PI * (s / 30.0 - 0.5);

  dc.SelectObject(TPen(TColor::LtRed, 2, PS_SOLID));

  dc.MoveTo(x,                   y);
  dc.LineTo(x + cr * cos(angle), y + cr * sin(angle));

  dc.RestorePen();

  // Draw the minute handle
  //
  cr -= spacing;

  angle = M_PI * ((m + floor(s / 15.0) / 4.0) / 30.0 - 0.5);

  dc.SelectObject(TPen(TColor::Black, 3, PS_SOLID));

  dc.MoveTo(x,                   y);
  dc.LineTo(x + cr * cos(angle), y + cr * sin(angle));

  dc.RestorePen();

  // Draw the hour handle
  //
  cr -= spacing;

  angle = M_PI * ((h + floor(m / 15.0) / 4.0) / 6.0 - 0.5);

  dc.SelectObject(TPen(TColor::LtMagenta, 4, PS_SOLID));

  dc.MoveTo(x,                   y);
  dc.LineTo(x + cr * cos(angle), y + cr * sin(angle));

  dc.RestorePen();
}

